# encoding: utf-8
require 'json'
require 'net/http'
require 'digest/md5'

module Geetest
  API_HOST = 'http://api.geetest.com'.freeze
  API_REGISTER_PATH = '/register.php'.freeze
  API_VALIDATE_PATH = '/validate.php'.freeze

  class SDK3
    attr_accessor :captcha_id, :private_key
    attr_reader :response_str
    attr_reader :session_key, :fn_challenge, :fn_validate, :fn_seccode

    def initialize(captcha_id, private_key)
      @captcha_id   = captcha_id
      @private_key  = private_key
      @response_str = ''

      @session_key  = 'gt_server_status'.freeze
      @fn_challenge = 'geetest_challenge'.freeze
      @fn_validate  = 'geetest_validate'.freeze
      @fn_seccode   = 'geetest_seccode'.freeze
    end

    def pre_process(user_id = nil)
      status, challenge = register(user_id)
      @response_str = make_response_format(status, challenge)
      status
    end

    def success_validate(challenge = '', validate = '', seccode = '', user_id = nil, gt = nil, data = '', user_info = '')
      return 0 if challenge.empty? || validate.empty? || seccode.empty?
      return 0 unless check_result(challenge, validate)

      validate_url = %(#{API_HOST}#{API_VALIDATE_PATH})
      query = {
        seccode: seccode,
        sdk: %(ruby_#{@sdk_version}),
        user_id: user_id,
        data: data,
        timestamp: Time.now.to_s,
        challenge: challenge,
        userinfo: user_info,
        captchaid: gt
      }
      response = Net::HTTP.post_form(URI(validate_url), query)
      response.body == md5_encode(seccode)
    end

    def failback_validate(challenge = '', validate = '', seccode = '')
      return 0 if challenge.empty? || validate.empty? || seccode.empty?

      validate_array = validate.split('_')
      encode_ans = validate_array[0]
      encode_fbii = validate_array[1]
      encode_igi = validate_array[2]

      decode_ans = decode_response(challenge, encode_ans)
      decode_fbii = decode_response(challenge, encode_fbii)
      decode_igi = decode_response(challenge, encode_igi)
      validate_fail_image(decode_ans, decode_fbii, decode_igi)
    end

    private

    def register(user_id)
      challenge = register_challenge(user_id)
      if challenge.length == 32
        challenge = md5_encode([challenge, private_key].join)
        [1, challenge]
      else
        [0, make_fail_challenge]
      end
    end

    def register_challenge(user_id)
      register_url = %(#{API_HOST}#{API_REGISTER_PATH}?gt=#{captcha_id})
      register_url = %(#{register_url}&user_id=#{user_id}) if user_id

      response = ::Net::HTTP.get_response(URI(register_url))
      response.is_a?(Net::HTTPSuccess) ? response.body : ''
    rescue => e
      e.message
    end

    def md5_encode(value)
      ::Digest::MD5.hexdigest(value.to_s)
    end

    def make_fail_challenge
      md5_encode(rand(99)) + md5_encode(rand(99))[0..2]
    end

    def make_response_format(success = 1, challenge = nil)
      {
        success: success,
        gt: @captcha_id,
        challenge: challenge || make_fail_challenge
      }.to_json
    end

    def check_result(origin, validate)
      validate == md5_encode(@private_key + 'geetest' + origin)
    end

    def decode_response(challenge, userresponse)
      return 0 if !userresponse || userresponse.length > 100

      numbers = [1, 2, 5, 10, 50]
      repeat_numbers = []
      h = {}
      challenge.chars.each_with_index do |char, i|
        continue if repeat_numbers.include?(char)

        h[char] = numbers[i % numbers.length]
        repeat_numbers.push(i)
      end
      res = 0
      userresponse.chars.each { |c| res += h.fetch(c, 0) }
      res - decode_rand_base(challenge)
    end

    def decode_rand_base(challenge)
      temp_array = challenge[32..-1].chars.map do |char|
        char_ascii = char.ord
        char_ascii - (char_ascii > 57 ? 87 : 48)
      end

      temp_array[0] * 36 + temp_array[1]
    end

    def validate_fail_image(ans, full_bg_index, img_grp_index)
      thread = 3
      full_bg_name = md5_encode(full_bg_index)[0..10]
      bg_name = md5_encode(img_grp_index)[10..20]

      answer_decode = 0.upto(9).map do |i|
        i.even? ? full_bg_name[i] : bg_name[i]
      end.join
      x_decode = answer_decode[4..-1]
      result = x_decode.to_i(16) % 200
      result = 40 if result < 40
      (ans - result).abs < thread
    end
  end
end
